﻿/*******************************************************************************
* Copyright (C) 2015 Intel Corp. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice,
*    this list of conditions and the following disclaimer.
*
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this list of conditions and the following disclaimer in the documentation
*    and/or other materials provided with the distribution.
*
*  - Neither the name of Intel Corp. nor the names of its
*    contributors may be used to endorse or promote products derived from this
*    software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL Intel Corp. OR THE CONTRIBUTORS
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

using System;

namespace FwUpdateAPI
{
    public enum HwGeneration
    {
        DSL4510_4410,       // Redwood Ridge
        DSL5520_5320,       // Falcon Ridge
        DSL5110,            // Thunderbolt LP - Win Ridge
        DSL6540_6340,       // Alpine Ridge
    }

    public enum HwType
    {
        _2Ports,
        _1Port
    }

    public class HwInfo
    {
        public HwGeneration Generation { get; set; }
        public HwType Type { get; set; }
    }

    /// <summary>
    /// Strategy class tree for abstracting the actual info extracting (reading), which can come
    /// from a controller or from an image file, for FwInfo class tree
    ///
    /// See ControllerFwInfoSource and FileFwInfoSource classes for the concrete classes
    /// </summary>
    public abstract class FwInfoSource
    {
        public abstract byte[] Read(uint offset, uint length);
        public abstract uint DigitalSectionOffset();

        public HwInfo Info { get; protected set; }

        public static HwInfo HwConfiguration(UInt16 controllerId)
        {
            switch (controllerId)
            {
                // DSL4510_4410
                case 0x1566:
                case 0x1567:
                    return new HwInfo { Generation = HwGeneration.DSL4510_4410, Type = HwType._1Port };
                case 0x1568:
                case 0x1569:
                    return new HwInfo { Generation = HwGeneration.DSL4510_4410, Type = HwType._2Ports };
                // DSL5520_DSL5320
                case 0x156A:
                case 0x156B:
                    return new HwInfo { Generation = HwGeneration.DSL5520_5320, Type = HwType._1Port };
                case 0x156C:
                case 0x156D:
                    return new HwInfo { Generation = HwGeneration.DSL5520_5320, Type = HwType._2Ports };
                // DSL5110
                case 0x157D:
                case 0x157E:
                    return new HwInfo { Generation = HwGeneration.DSL5110, Type = HwType._1Port };
                // DSL6540_6340
                case 0x1577:
                case 0x1578:
                    return new HwInfo { Generation = HwGeneration.DSL6540_6340, Type = HwType._2Ports };
                case 0x1575:
                case 0x1576:
                    return new HwInfo { Generation = HwGeneration.DSL6540_6340, Type = HwType._1Port };
                default:
                    throw new TbtException(TbtStatus.SDK_UNKNOWN_CHIP);
            }
        }

        /// <summary>
        /// Reads the HW generation and type (port count) from FW (from controller or FW image)
        /// </summary>
        /// <returns>
        /// HWInfo object with generation and type (port count) info in regular case
        /// </returns>
        /// <exception>Throws in case of unknown device ID</exception>
        /// <remarks>
        /// We call it from each derived class c-tor, not from base class c-tor, because the use of
        /// abstract functions Read() and DigitalSectionOffset(), which may need the derived class
        /// initialized
        /// </remarks>
        protected HwInfo GetHwConfiguration()
        {
            var deviceId = new FwLocation { Offset = DigitalSectionOffset() + 0x5, Length = 2 };
            var resultBuffer = Read(deviceId.Offset, deviceId.Length);
            return HwConfiguration(BitConverter.ToUInt16(resultBuffer, 0));
        }
    }
}
